@@ -38,23 +38,7 @@
 #include <string.h>
 #include <glib/gi18n-lib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtkalignment.h> 
-#include <gtk/gtkbox.h> 
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcellrenderertoggle.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkicontheme.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkvbox.h>
+#include <gtk/gtk.h>
 #include <gio/gio.h>
 
 struct _NautilusMimeApplicationChooserDetails {
@@ -599,9 +583,9 @@ set_extension_and_description (NautilusMimeApplicationChooser *chooser,
 			       const char *extension,
 			       const char *mime_type)
 {
-	chooser->details->extension = g_strdup (extension);
 	if (extension != NULL &&
 	    g_content_type_is_unknown (mime_type)) {
+		chooser->details->extension = g_strdup (extension);
 		    chooser->details->content_type = g_strdup_printf ("application/x-extension-%s", extension);
 		    /* the %s here is a file extension */
 		    chooser->details->type_description =
